



swdigridder(1)                                                  swdigridder(1)



NAME
       swdigridder - calculate frequency grids from SWDI data

SYNOPSIS
       java -jar swdigridder.jar [OPTIONS] OUTPUT_FILE INPUT_FILE [...]

DESCRIPTION
       SwdiGridder reads csv files containing SWDI data and produces a gridded
       NetCDF file in which the value assigned to each grid cell is the number
       of features in the input data that fall within that grid cell.

       SWDI  stands for Severe Weather Data Inventory, a project at NOAA's Na-
       tional Climatic Data Center (NCDC) that is focused on providing  conve-
       nient  access  to  multiple  sources of severe weather data.  Currently
       most SWDI data consists of point features extracted from the Level  III
       NEXRAD radar products tvs, mesocyclone, hail, and storm structure.  The
       SWDI project is designed to handle data from multiple sources,  though,
       and  this program and its documentation are written in general terms to
       work with any SWDI data.

       SWDI data files in the format expected by this program can be download-
       ed via ftp from ftp://ftp.ncdc.noaa.gov/pub/data/swdi.  SwdiGridder can
       read either plain or gzipped input files; if an input file's name  ends
       with  ".gz",  it  is assumed to be gzipped, and SwdiGridder will uncom-
       press the file as it reads it.

       The output file written by SwdiGridder is a gridded NetCDF  file  which
       is readable by any of the many software tools that can read NetCDF.

       Note  that swdigridder.jar is a self-contained executable jar file; you
       do not need to set any classpath in order to  use  it.   (It  uses  the
       "one-jar" system from http://one-jar.sourceforge.net to embed a copy of
       the NetCDF jar file inside itself.)  The main class that it invokes  is
       gov.noaa.ncdc.swdi.gridder.SwdiGridder.


INPUT FILE FORMAT
       If  you  download SWDI data from ftp://ftp.ncdc.noaa.gov/pub/data/swdi,
       you should be able to run SwdiGridder on it without having  to  concern
       yourself with the data format.

       If you want to modify SwdiGridder to handle the data differently, or if
       you want to write your own software to process the data, here is a  de-
       scription of the data format.

       SwdiGridder input files contain one data point per line; each line con-
       sists of a sequence of comma-separated values.   In  keeping  with  GIS
       terminology,  each data point is called a "feature", and the values as-
       sociated with each feature are called its "attributes".

       Any line that starts with a '#' character (which  must  appear  as  the
       first  character  on  the  line)  will  be ignored, with one exception.
       There should be exactly one line near the top of the file  that  starts
       with "#ZTIME", and SwdiGridder treats this line as a "header" line that
       contains a comma-separated list of attribute names.

       SwdiGridder expects the first three attributes on each data line to  be
       ztime,  longitude,  and  latitude, in that order.  The order of the re-
       maining attributes does not matter, but it should match the order given
       by the header line.  If multiple input files are given, SwdiGridder on-
       ly reads the header line from the first file.  It assumes that the  at-
       tribute order is the same for all files.


TIME BINNING
       By default, SwdiGridder does something called "time binning" on the in-
       put data as a crude way to reduce the possibility of a single  meteoro-
       logical feature being counted more than once.  (The same meteorological
       feature often appears in SWDI data more than once, because it  was  ob-
       served  multiple  times,  or  from multiple sites.)  In time binning, a
       "time binning interval" is chosen, and when the data  is  processed  to
       produce grid cell counts, only the first feature in each grid cell dur-
       ing each time binning interval is counted.  The default binning  inter-
       val is 15 minutes, but this can be modified with the -b option.

       SwdiGridder's  time  binning  algorithm depends on the input data being
       sorted in ascending chronological order.  The SWDI files available from
       ftp://ftp.ncdc.noaa.gov/pub/data/swdi  have  been  written in this way.
       If the input data is not in chronological order, the time binning algo-
       rithm  will get confused and the results will be mostly likely meaning-
       less.


FILTERING BY ATTRIBUTE
       By default, SwdiGridder ignores the values of all attributes other than
       ztime, longitude, and latitude.  The -f option lets you specify a "fil-
       ter" expression that determines which features to count  based  on  at-
       tribute  values.  This  filter expression should be a sequence of rela-
       tional expressions separated by the boolean "&&" operator.  Each  rela-
       tional  expression is of the form "ATTR OP VALUE", where ATTR is an at-
       tribute name, OP is one of the operators <, <=, ==, !=, >, or  >=,  and
       VALUE  is  either  a  number,  or  a  string enclosed in single quotes.
       String values are only allowed for the == and != operator.   The  whole
       filtering  expression  should be delimited by double quotes on the com-
       mand line, so that it sappears as one argument after the -f option.

       Note that the relational expressions must be joined to each other  with
       the boolean AND operator "&&".  Currently only the AND operator is sup-
       ported; this might be extended to incude OR ("||") in the future.

       The code that parses filter expressions is not very  smart  --  it  re-
       quires every token (attribute name, operator, or value) to be separated
       from the next by whitespace.  So be sure to write "MAXSIZE > 0" and not
       "MAXSIZE>0", for example.

       See the EXAMPLES section below for some examples.

       The filter expression indicates which feature should be counted (as op-
       posed to which ones should be ignored).

       Filtering happens before time binning, so that if a feature  is  elimi-
       nated  by  the filter expression, it does affect the value for its grid
       cell at all.


OPTIONS
       -s M   Grid cell size, in 100ths of a degree.  The default  is  5.   Be
              aware that smaller grid cell sizes result in larger grids, which
              in turn require more memory to compute.  You  can  increase  the
              amount  of  memory  available to java programs by using the -Xms
              and -Xmx arguments to  the  "java"  command.   I  typically  use
              "-Xms512m -Xmx1024m".


       -e WEST EAST SOUTH NORTH
              Compute  a  frequency  grid for the geographic region with these
              bounds, given in signed decimal degrees.  Any  features  outside
              this  region will be completely ignored.  The default is -125 65
              24 50.


       -b N   Change the time binning interval to N minutes;  the  default  is
              15.   A value of 0 or less disables time binning completely; all
              features will be counted.


       -f FILTER_EXPRESSION
              Specifies an expression for attribute filtering.



EXAMPLES
       java -Xms512m -Xmx1024m gov.noaa.ncdc.swdi.gridder.SwdiGridder -s 1  -f
       "CELL_ID != '??'" tvs.nc tvs-*.csv.gz
              Process all the tvs files in the current directory to produce  a
              grid  with  cell  size  0.01 degree, only couning features whose
              CELL_ID attribute is not equal to the string '??'.  The -Xms512m
              -Xmx1024m options increase the amount of memory available to the
              program; this is needed on my system with a grid this large.


       java gov.noaa.ncdc.swdi.gridder.SwdiGridder -f "MAXSIZE > 0 && PROB  ==
       100"  hail.nc hail-*.csv.gz
              Process all the hail files in the current directory, only  coun-
              ing features for which MAXSIZE > 0 and PROB == 0.



FEEDBACK AND SUPPORT
       If you use this program, or write your own software to process SWDI da-
       ta, I would be very interested in hearing about  your  experience,  and
       I'll be happy to provide any support that I can.  You can contact me at
       the address or number below.


AUTHOR
       Mark Phillips
       Mark.Phillips@noaa.gov
       828-271-4487




                               February 28, 2008                swdigridder(1)
