/*
 * Author:
 *    Mark Phillips
 *    University of North Carolina Asheville /  National Climatic Data Center
 *    Mark.Phillips@noaa.gov
 *    828-271-4487
 */

package gov.noaa.ncdc.swdi.gridder;

import java.awt.Color;

public class ColorRamp {

	protected int numClasses;
	protected int numClassesMinusOne;
	protected Color colors[];
	protected double values[];
	private int i;

	private int counts[];
	private boolean counting;

	public ColorRamp(double values[],
					 int rMin, int gMin, int bMin, int rMax, int gMax, int bMax) throws Exception {
		int numClasses = values.length - 1;
		Color colors[] = new Color[numClasses];
		for (int i=0; i<numClasses; ++i) {
			double s = i/((double)numClasses-1);
			double t = 1.0 - s;
			int r = (int)(Math.round(t * rMin + s * rMax));
			int g = (int)(Math.round(t * gMin + s * gMax));
			int b = (int)(Math.round(t * bMin + s * bMax));
			colors[i] = new Color(r,g,b);
		}

		setValuesAndColors(values, colors);
	}

	public ColorRamp(double min_value, double max_value,
					 int numClasses,
					 int rMin, int gMin, int bMin, int rMax, int gMax, int bMax) throws Exception {
		double classLength = (max_value - min_value) / numClasses;
		Color colors[] = new Color[numClasses];
		double values[] = new double[numClasses+1];
		for (int i=0; i<numClasses; ++i) {
			values[i] = min_value + i * classLength;
			double s = i/((double)numClasses-1);
			double t = 1.0 - s;
			int r = (int)(Math.round(t * rMin + s * rMax));
			int g = (int)(Math.round(t * gMin + s * gMax));
			int b = (int)(Math.round(t * bMin + s * bMax));
			colors[i] = new Color(r,g,b);
		}
		values[numClasses] = max_value;

		setValuesAndColors(values, colors);
	}

	private void setValuesAndColors(double values[], Color colors[]) throws Exception {
		if (values.length != colors.length + 1) {
			throw new Exception("values array must have length one more than length of colors array");
		}
		this.values   = values;
		this.colors   = colors;
		this.numClasses = colors.length;
		this.numClassesMinusOne = this.numClasses - 1;
		this.counts = null;
		this.counting = false;
	}

	public int getNumClasses() { return numClasses; }
	public Color[] getColors() { return colors; }
	public double[] getValues() { return values; }

	public int valueToClass(double v) {
		if (v <= values[1]) {
			if (counting) { ++counts[0]; }
			return 0;
		}
		for (i=1; i<=numClassesMinusOne; ++i) {
			if (v <= values[i+1]) {
				if (counting) { ++counts[i]; }
				return i;
			}
		}
		if (counting) { ++counts[numClassesMinusOne]; }
		return numClassesMinusOne;
	}

	public Color valueToColor(double v) {
		return colors[valueToClass(v)];
	}

	public Color classToColor(int i) {
		return colors[i];
	}

	public void startCounts() {
		counts = new int[numClasses];
		for (int i=0; i<numClasses; ++i) {
			counts[i] = 0;
		}
		counting = true;
	}
	public void stopCounts() {
		counting = false;
	}
	public int[] getCounts() { return counts; }

}

