package gov.noaa.ncdc.swdi.gridder;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.font.GlyphVector;
import java.awt.geom.Rectangle2D;
import java.awt.font.FontRenderContext;
import java.util.List;
import java.util.ArrayList;    
import java.awt.Font;

public class ColorRampLegend {

    private ColorRamp colorRamp;
    private int legendBoxSize;
    private int legendVPad;

    private int legendWidth;
    private int legendHeight;

    public ColorRampLegend(ColorRamp  colorRamp,
                           int        legendBoxSize,
                           int        legendVPad) {
        this.colorRamp          = colorRamp;
        this.legendBoxSize      = legendBoxSize;
        this.legendVPad         = legendVPad;

        this.legendHeight = legendBoxSize * colorRamp.getNumClasses() + legendVPad * (colorRamp.getNumClasses() - 1);
        this.legendWidth  = legendBoxSize;
    }

    public int getWidth() { return this.legendWidth; }
    public int getHeight() { return this.legendHeight; }

    public void render(Graphics2D g2d,
                       int x0, int y0,
                       int anchorX, int anchorY) { 
       Color oldColor = g2d.getColor();

        int legendLeft   = (int)Math.round(x0 - (anchorX + 1)*legendWidth/2.0);
        int legendTop    = (int)Math.round(y0 - (1 - anchorY)*legendHeight/2.0);
        int legendRight  = legendLeft + legendWidth;
        int legendBottom = legendTop + legendHeight;

        /* draw the boxes, starting at the bottom */
		for (int i=0; i<colorRamp.getNumClasses(); ++i) {
            try {
                g2d.setColor( colorRamp.classToColor(i) );
                g2d.fillRect(legendLeft, legendBottom-(i+1)*legendBoxSize-i*legendVPad, legendBoxSize, legendBoxSize);
            } catch (Exception e) {}
		}

        g2d.setColor(oldColor);
    }

}
