package gov.noaa.ncdc.swdi.gridder;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.URL;

import javax.imageio.ImageIO;

import org.geotools.feature.SchemaException;
import org.geotools.geometry.GeneralEnvelope;
import org.geotools.geometry.jts.ReferencedEnvelope;
import org.geotools.map.DefaultMapContext;
import org.geotools.map.MapContext;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.geotools.renderer.lite.StreamingRenderer;
import org.opengis.feature.IllegalAttributeException;
import org.opengis.referencing.operation.TransformException;

import com.vividsolutions.jts.geom.Envelope;


public class MakeImage {

    public static void main(String[] args) {

        try {
			createMap();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }




    public static void createMap()
		throws IOException, IllegalAttributeException, SchemaException, TransformException {

        int width  = 600;
        int height = 260;
        Dimension imageDimension = new Dimension(width, height);
        BufferedImage outputImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);  
        
        // draw map background as 'ocean' color
        Graphics2D g2d = (Graphics2D)outputImage.getGraphics();
        g2d.setColor(new Color(190, 232, 254));
        g2d.fillRect(0, 0, width, height);

        // 6. init map context and GTHelper
        MapContext context = new DefaultMapContext(DefaultGeographicCRS.WGS84);
        
        // draw white background
        GTHelper.addShapefileToContext(
                context, 
                GTHelper.class.getResource("/shapefiles/countries-noUSA.shp"), 
                GTHelper.class.getResource("/styles/whiteBackground.sld"), 
                "Countries BG");
        
        
        GTHelper.addShapefileToContext(
                context, 
                GTHelper.class.getResource("/shapefiles/states.shp"), 
                GTHelper.class.getResource("/styles/whiteBackground.sld"), 
                "States BG");
        
        GTHelper.addShapefileToContext(
                context, 
                GTHelper.class.getResource("/shapefiles/lakes.shp"), 
                GTHelper.class.getResource("/styles/waterBackground.sld"), 
                "Lakes BG");

        ReferencedEnvelope refEnv =
			new ReferencedEnvelope(new Envelope(-125.0,
												-67.0,
												20.0,
												54.0),
												context.getCoordinateReferenceSystem());


        // 8. add states background to context
        GTHelper.addShapefileToContext(
                context, 
                GTHelper.class.getResource("/shapefiles/countries-noUSA.shp"), 
                GTHelper.class.getResource("/styles/countriesStyleFGMosaic.sld"), 
                "Countries FG");
        
        
        GTHelper.addShapefileToContext(
                context, 
                GTHelper.class.getResource("/shapefiles/states.shp"), 
                GTHelper.class.getResource("/styles/stateStyleFGMosaic.sld"), 
                "States FG");

        
        // 9. init renderer
        StreamingRenderer renderer = new StreamingRenderer();

        renderer.setContext(context);
        
        
        // 11. paint the context onto the blank image
        renderer.paint(
                (Graphics2D)outputImage.getGraphics(), 
                new Rectangle(outputImage.getWidth(), outputImage.getHeight()),
                GTHelper.adjustGeographicBounds(outputImage, refEnv));


        // Draw NOAA Logo
        URL localnoaaURL = MakeImage.class.getResource("/images/NOAA2small.png");
        BufferedImage noaaBI = ImageIO.read(localnoaaURL);
        g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));
        g2d.drawImage(noaaBI, 1, height-noaaBI.getHeight()-1, null);
        
        // 12. write any legend or title info
        Font font = new Font("Verdana", Font.BOLD, 12);
        g2d.setFont(font);
        
        String text = "SWDI Climatology: bla bla bla bla bla";
        FontRenderContext fc = g2d.getFontRenderContext();
        GlyphVector   glyphs = g2d.getFont().createGlyphVector(fc, text);
        Rectangle2D   bounds = glyphs.getVisualBounds();
        
        int textWidth = (int)bounds.getWidth();
        int textHeight = (int)bounds.getHeight();
        int xBuffer = 10;
        int yBuffer = 4;
        int bottomInsetHeight = 8;
        
        g2d.setColor(new Color(255, 255, 255, 160));
        g2d.fillRoundRect(width/2-(textWidth/2)-xBuffer, height-textHeight-yBuffer-bottomInsetHeight, 
                textWidth+(xBuffer*2), textHeight+(yBuffer*2), 5, 5);
        
        g2d.setColor(Color.BLACK);
        g2d.drawString(text, width/2-(textWidth/2), height-bottomInsetHeight);

        
		ImageIO.write(outputImage, "png", new File("foo.png"));

    }


	/*
    private static GeneralEnvelope readWorldFile(URL wrdUrl, Dimension imageDim) throws IOException {
        // 1. read world file
        //      The parameters found in the file should be as follows:
        //
        //          1. size of pixel in x direction
        //          2. rotation term for row
        //          3. rotation term for column
        //          4. size of pixel in y direction
        //          5. x coordinate of centre of upper left pixel in map units
        //          6. y coordinate of centre of upper left pixel in map units




        BufferedReader br = new BufferedReader(new InputStreamReader(wrdUrl.openStream()));
        String inputLine;
        String xPixelSize = null;
        String yPixelSize = null;
        String rotRow = null;
        String rotColumn = null;
        String ulCenterX = null;
        String ulCenterY = null;

        int count = 0;
        while ((inputLine = br.readLine()) != null) {
            if (count == 0) {
                xPixelSize = inputLine; 
            }
            else if (count == 1) {
                rotRow = inputLine; 
            }
            else if (count == 2) {
                rotColumn = inputLine; 
            }
            else if (count == 3) {
                yPixelSize = inputLine; 
            }
            else if (count == 4) {
                ulCenterX = inputLine; 
            }
            else if (count == 5) {
                ulCenterY = inputLine; 
            }

            count++;
        }
        br.close();


        // 2. create envelope (geographic bounds)            
        double x = Double.parseDouble(ulCenterX);
        double geoWidth = Math.abs(Double.parseDouble(xPixelSize)*imageDim.getWidth());
        double y = Double.parseDouble(ulCenterY);
        double geoHeight = Math.abs(Double.parseDouble(yPixelSize)*imageDim.getHeight());

        GeneralEnvelope env = new GeneralEnvelope(new Rectangle2D.Double(x, y-geoHeight, geoWidth, geoHeight));

        return env;
    }
	*/




}
