import gov.noaa.ncdc.swdi.gridder.ColorRamp;
import gov.noaa.ncdc.swdi.gridder.DomainGridder;
import java.awt.Color;
import java.util.List;

import org.junit.Assert;
import static org.junit.Assert.*;
import junit.framework.JUnit4TestAdapter;
import org.junit.Before;
import org.junit.Test;

public class TestSuite {

	public static junit.framework.Test suite() {
		 return new JUnit4TestAdapter(TestSuite.class);
	}

	private static String join(List objects, String glue) {
		StringBuffer sb = new StringBuffer();
		boolean first = true;
		for (Object o : objects) {
			if (!first) { sb.append(glue); }
			first = false;
			sb.append(o.toString());
		}
		return sb.toString();
	}

	@Test
		public void colorRampTest() {

		try {
			//  |-------|-------|-------|
			// 0.0     1.0     2.0     3.0
			//     i=0     i=1     i=2
			ColorRamp colorRamp = new ColorRamp(0.0, 3.0, 3,
												240, 240, 240,
												200,  80,   0);
			assertEquals( "value -0.5 maps to index 0",   0, colorRamp.valueToIndex(-0.5));
			assertEquals( "value 0.0 maps to index 0",   0, colorRamp.valueToIndex(0.0));
			assertEquals( "value 0.5 maps to index 0",   0, colorRamp.valueToIndex(0.5));
			assertEquals( "value 1.0 maps to index 0",   0, colorRamp.valueToIndex(1.0));

			assertEquals( "value 1.5 maps to index 1",   1, colorRamp.valueToIndex(1.5));
			assertEquals( "value 2.0 maps to index 1",   1, colorRamp.valueToIndex(2.0));

			assertEquals( "value 2.5 maps to index 2",   2, colorRamp.valueToIndex(2.5));
			assertEquals( "value 3.0 maps to index 2",   2, colorRamp.valueToIndex(3.0));
			assertEquals( "value 3.5 maps to index 2",   2, colorRamp.valueToIndex(3.5));

			Color c = colorRamp.indexToColor(0);
			assertEquals("color 0 red component is 240", 240, c.getRed());
			assertEquals("color 0 green component is 240", 240, c.getGreen());
			assertEquals("color 0 blue component is 240", 240, c.getBlue());

			c = colorRamp.indexToColor(1);
			assertEquals("color 1 red component is 220", 220, c.getRed());
			assertEquals("color 1 green component is 160", 160, c.getGreen());
			assertEquals("color 1 blue component is 120", 120, c.getBlue());

			c = colorRamp.indexToColor(2);
			assertEquals("color 2 red component is 200", 200, c.getRed());
			assertEquals("color 2 green component is 80", 80, c.getGreen());
			assertEquals("color 2 blue component is 0", 0, c.getBlue());

			//assertEquals(0,     r.getLeft(),        0.00001);
			//			assertEquals(1,     r.getRight(),       0.00001);
			//			assertEquals(false, r.getClosedLeft());
			//			assertEquals(false, r.getClosedRight());
			//			assertEquals(false, r.getMapped());
			//			assertEquals(false, r.contains(0.0));
			//			assertEquals(false, r.contains(1.0));
			//			for (double x = 0.1; x < 1.0; x += 0.1) {
			//				assertEquals(true, r.contains(x));
			//			}
		} catch (Exception e) {
			Assert.fail(e.toString());
		}

	}

//	@Test
//		public void test2() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("[0,1)");
//			Grib2Shp.MappedRange r = ranges.get(0);
//			assertEquals(0,     r.getLeft(),        0.00001);
//			assertEquals(1,     r.getRight(),       0.00001);
//			assertEquals(true,  r.getClosedLeft());
//			assertEquals(false, r.getClosedRight());
//			assertEquals(false, r.getMapped());
//			assertEquals(true,  r.contains(0.0));
//			assertEquals(false, r.contains(1.0));
//			for (double x = 0.1; x < 1.0; x += 0.1) {
//				assertEquals(true, r.contains(x));
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test3() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("(0,1]");
//			Grib2Shp.MappedRange r = ranges.get(0);
//			assertEquals(0,     r.getLeft(),        0.00001);
//			assertEquals(1,     r.getRight(),       0.00001);
//			assertEquals(false, r.getClosedLeft());
//			assertEquals(true,  r.getClosedRight());
//			assertEquals(false, r.getMapped());
//			assertEquals(false, r.contains(0.0));
//			assertEquals(true,  r.contains(1.0));
//			for (double x = 0.1; x < 1.0; x += 0.1) {
//				assertEquals(true, r.contains(x));
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test4() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("[0,1]");
//			Grib2Shp.MappedRange r = ranges.get(0);
//			assertEquals(0,     r.getLeft(),        0.00001);
//			assertEquals(1,     r.getRight(),       0.00001);
//			assertEquals(true,  r.getClosedLeft());
//			assertEquals(true,  r.getClosedRight());
//			assertEquals(false, r.getMapped());
//			assertEquals(true,  r.contains(0.0));
//			assertEquals(true,  r.contains(1.0));
//			for (double x = 0.1; x < 1.0; x += 0.1) {
//				assertEquals(true, r.contains(x));
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test5() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("[0,1]=L");
//			Grib2Shp.MappedRange r = ranges.get(0);
//			assertEquals(0,     r.getLeft(),        0.00001);
//			assertEquals(1,     r.getRight(),       0.00001);
//			assertEquals(true,  r.getClosedLeft());
//			assertEquals(true,  r.getClosedRight());
//			assertEquals(true,  r.getMapped());
//			assertEquals(0,     r.getMappedValue(), 0.00001);
//			assertEquals(true,  r.contains(0.0));
//			assertEquals(true,  r.contains(1.0));
//			for (double x = 0.1; x < 1.0; x += 0.1) {
//				assertEquals(true, r.contains(x));
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test6() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("[0,1]=U");
//			Grib2Shp.MappedRange r = ranges.get(0);
//			assertEquals(0,     r.getLeft(),        0.00001);
//			assertEquals(1,     r.getRight(),       0.00001);
//			assertEquals(true,  r.getClosedLeft());
//			assertEquals(true,  r.getClosedRight());
//			assertEquals(true,  r.getMapped());
//			assertEquals(1,     r.getMappedValue(), 0.00001);
//			assertEquals(true,  r.contains(0.0));
//			assertEquals(true,  r.contains(1.0));
//			for (double x = 0.1; x < 1.0; x += 0.1) {
//				assertEquals(true, r.contains(x));
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test7() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("[0,1]=I");
//			Grib2Shp.MappedRange r = ranges.get(0);
//			assertEquals(0,     r.getLeft(),        0.00001);
//			assertEquals(1,     r.getRight(),       0.00001);
//			assertEquals(true,  r.getClosedLeft());
//			assertEquals(true,  r.getClosedRight());
//			assertEquals(true,  r.getMapped());
//			assertEquals(0,     r.getMappedValue(), 0.00001);
//			assertEquals(true,  r.contains(0.0));
//			assertEquals(true,  r.contains(1.0));
//			for (double x = 0.1; x < 1.0; x += 0.1) {
//				assertEquals(true, r.contains(x));
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test8() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("(0,10]=I (10,20]=I");
//			assertEquals(2, ranges.size());
//			{
//				Grib2Shp.MappedRange r = ranges.get(0);
//				assertEquals(0,     r.getLeft(),        0.00001);
//				assertEquals(10,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(0,     r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(0.0));
//				assertEquals(true,  r.contains(10.0));
//				for (double x = 0.1; x < 10.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(1);
//				assertEquals(10,    r.getLeft(),        0.00001);
//				assertEquals(20,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(1,     r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(10.0));
//				assertEquals(true,  r.contains(20.0));
//				for (double x = 10.1; x < 20.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test9() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("(5,10]=L (10,20]=L");
//			assertEquals(2, ranges.size());
//			{
//				Grib2Shp.MappedRange r = ranges.get(0);
//				assertEquals(5,     r.getLeft(),        0.00001);
//				assertEquals(10,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(5,     r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(5.0));
//				assertEquals(true,  r.contains(10.0));
//				for (double x = 5.1; x < 10.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(1);
//				assertEquals(10,    r.getLeft(),        0.00001);
//				assertEquals(20,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(10,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(10.0));
//				assertEquals(true,  r.contains(20.0));
//				for (double x = 10.1; x < 20.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test10() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("(5,10]=U (10,20]=U");
//			assertEquals(2, ranges.size());
//			{
//				Grib2Shp.MappedRange r = ranges.get(0);
//				assertEquals(5,     r.getLeft(),        0.00001);
//				assertEquals(10,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(10,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(5.0));
//				assertEquals(true,  r.contains(10.0));
//				for (double x = 5.1; x < 10.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(1);
//				assertEquals(10,    r.getLeft(),        0.00001);
//				assertEquals(20,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(20,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(10.0));
//				assertEquals(true,  r.contains(20.0));
//				for (double x = 10.1; x < 20.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test11() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("(5,10]=U (10,20]=U (20:5:30]=U");
//			assertEquals(4, ranges.size());
//			{
//				Grib2Shp.MappedRange r = ranges.get(0);
//				assertEquals(5,     r.getLeft(),        0.00001);
//				assertEquals(10,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(10,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(5.0));
//				assertEquals(true,  r.contains(10.0));
//				for (double x = 5.1; x < 10.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(1);
//				assertEquals(10,    r.getLeft(),        0.00001);
//				assertEquals(20,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(20,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(10.0));
//				assertEquals(true,  r.contains(20.0));
//				for (double x = 10.1; x < 20.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(2);
//				assertEquals(20,    r.getLeft(),        0.00001);
//				assertEquals(25,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(25,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(20.0));
//				assertEquals(true,  r.contains(25.0));
//				for (double x = 20.1; x < 25.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(3);
//				assertEquals(25,    r.getLeft(),        0.00001);
//				assertEquals(30,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(30,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(25.0));
//				assertEquals(true,  r.contains(30.0));
//				for (double x = 25.1; x < 30.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test12() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("(5,10]=L (10,20]=L (20:5:30]=L");
//			assertEquals(4, ranges.size());
//			{
//				Grib2Shp.MappedRange r = ranges.get(0);
//				assertEquals(5,     r.getLeft(),        0.00001);
//				assertEquals(10,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(5,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(5.0));
//				assertEquals(true,  r.contains(10.0));
//				for (double x = 5.1; x < 10.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(1);
//				assertEquals(10,    r.getLeft(),        0.00001);
//				assertEquals(20,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(10,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(10.0));
//				assertEquals(true,  r.contains(20.0));
//				for (double x = 10.1; x < 20.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(2);
//				assertEquals(20,    r.getLeft(),        0.00001);
//				assertEquals(25,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(20,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(20.0));
//				assertEquals(true,  r.contains(25.0));
//				for (double x = 20.1; x < 25.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(3);
//				assertEquals(25,    r.getLeft(),        0.00001);
//				assertEquals(30,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(25,    r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(25.0));
//				assertEquals(true,  r.contains(30.0));
//				for (double x = 25.1; x < 30.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//
//	@Test
//		public void test13() {
//
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList("(5,10] (10,20]=I (20:5:30]=I");
//			assertEquals(4, ranges.size());
//			{
//				Grib2Shp.MappedRange r = ranges.get(0);
//				assertEquals(5,     r.getLeft(),        0.00001);
//				assertEquals(10,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(false, r.getMapped());
//				assertEquals(0,     r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(5.0));
//				assertEquals(true,  r.contains(10.0));
//				for (double x = 5.1; x < 10.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(1);
//				assertEquals(10,    r.getLeft(),        0.00001);
//				assertEquals(20,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(1,     r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(10.0));
//				assertEquals(true,  r.contains(20.0));
//				for (double x = 10.1; x < 20.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(2);
//				assertEquals(20,    r.getLeft(),        0.00001);
//				assertEquals(25,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(2,     r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(20.0));
//				assertEquals(true,  r.contains(25.0));
//				for (double x = 20.1; x < 25.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//			{
//				Grib2Shp.MappedRange r = ranges.get(3);
//				assertEquals(25,    r.getLeft(),        0.00001);
//				assertEquals(30,    r.getRight(),       0.00001);
//				assertEquals(false, r.getClosedLeft());
//				assertEquals(true,  r.getClosedRight());
//				assertEquals(true,  r.getMapped());
//				assertEquals(3,     r.getMappedValue(), 0.00001);
//				assertEquals(false, r.contains(25.0));
//				assertEquals(true,  r.contains(30.0));
//				for (double x = 25.1; x < 30.0; x += 0.1) {
//					assertEquals(true, r.contains(x));
//				}
//			}
//		} catch (Exception e) {
//			Assert.fail(e.toString());
//		}
//
//	}
//
//	@Test
//		public void test14() {
//
//		String input = "(-99999, -1]=U  [-1, 0)=U  (0:50:150]=U";
//		try {
//			List<Grib2Shp.MappedRange> ranges = Grib2Shp.MappedRange.parseMappedRangeList(input);
//			assertEquals("(-99999.0, -1.0]=-1.0; [-1.0, 0.0)=0.0; (0.0, 50.0]=50.0; (50.0, 100.0]=100.0; (100.0, 150.0]=150.0",
//						 join(ranges, "; "));
//		} catch (Exception e) {
//			Assert.fail();
//		}
//
//	}
//
//
//	@Test
//		public void test100() {
//
//		try {
//			String args[] = {
//				"-a",
//				"precip",
//				"test/data/ST4.2007070112.24h",
//				"Total_precipitation",
//				"(-99999, -1]  (-1, 0]=U (0:50:1000]=U",
//				"rain.shp"
//			};
//			Grib2Shp.main(args);
//		} catch (Exception e) {
//			e.printStackTrace();
//			Assert.fail(e.getMessage());
//		}
//
//	}
//	

}

